package com.huertian.jinduguanli.service;

import com.huertian.jinduguanli.common.ApiResponse;
import com.huertian.jinduguanli.common.ErrorCode;
import com.huertian.jinduguanli.dto.*;
import com.huertian.jinduguanli.entity.Department;
import com.huertian.jinduguanli.entity.User;
import com.huertian.jinduguanli.repository.DepartmentRepository;
import com.huertian.jinduguanli.repository.UserRepository;
import com.huertian.jinduguanli.security.service.JwtService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final JwtService jwtService;
    private final DepartmentRepository departmentRepository;
    private static final Logger logger = LoggerFactory.getLogger(UserService.class);

    @Autowired
    public UserService(JwtService jwtService, PasswordEncoder passwordEncoder, UserRepository userRepository,
            DepartmentRepository departmentRepository) {
        this.jwtService = jwtService;
        this.passwordEncoder = passwordEncoder;
        this.userRepository = userRepository;
        this.departmentRepository = departmentRepository;
    }

    @Override
    public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {
        Optional<User> user = userRepository.findByEmail(email);
        if (user.isEmpty()) {
            throw new UsernameNotFoundException("User not found with email: " + email);
        }
        return user.get();
    }

    public ApiResponse<LoginResponse> login(UserLoginRequest request) {
        Optional<User> userOpt = userRepository.findByEmail(request.getEmail());
        if (userOpt.isEmpty() || userOpt.get().getStatus() != 1) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND_OR_DISABLED, "用户不存在或已禁用", null);
        }

        User user = userOpt.get();
        if (!passwordEncoder.matches(request.getPassword(), user.getPassword())) {
            return new ApiResponse<>(ErrorCode.PASSWORD_INCORRECT, "密码错误", null);
        }

        String token = jwtService.generateToken(user);
        return ApiResponse.success(new LoginResponse(token));
    }

    public ApiResponse<Void> createUser(CreateUserRequest request) {
        // 检查邮箱是否已存在
        if (userRepository.findByEmail(request.getEmail()).isPresent()) {
            return new ApiResponse<>(ErrorCode.EMAIL_EXISTS, "邮箱已存在", null);
        }

        User user = new User();
        user.setEmail(request.getEmail());
        user.setUsername(request.getUsername());
        user.setPassword(passwordEncoder.encode(request.getPassword()));
        user.setDepartmentId(request.getDepartmentId());
        user.setRoles(request.getRoles());
        user.setJobs(request.getJobs());
        user.setCreatorId(request.getCreatorId());
        user.setStatus(1);
        // 时间戳由 @PrePersist 处理，这里不需要手动设置

        userRepository.save(user);

        return ApiResponse.success();
    }

    public ApiResponse<User> getCurrentUser(String token) {
        String email = jwtService.extractUsername(token);
        logger.info("从token中提取的邮箱: {}", email);
        Optional<User> userOpt = userRepository.findByEmail(email);
        if (userOpt.isEmpty()) {
            logger.warn("用户不存在: {}", email);
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        User user = userOpt.get();
        logger.info("找到用户: {}", user.getUsername());
        user.setPassword(null);
        return ApiResponse.success(user);
    }

    public ApiResponse<String> disableUser(Long userId) {
        Optional<User> userOpt = userRepository.findById(userId);

        if (userOpt.isEmpty()) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        User user = userOpt.get();
        user.setStatus(0);
        userRepository.save(user);

        return ApiResponse.success("用户已禁用");
    }

    public ApiResponse<UserPageResponse> listUsers(Integer currentPage, Integer pageSize) {
        // 参数校验
        if (currentPage < 1) {
            currentPage = 1;
        }
        if (pageSize < 1) {
            pageSize = 10;
        }

        // 查询数据
        Page<User> userPage = userRepository.findAll(PageRequest.of(currentPage - 1, pageSize));

        // 处理返回数据
        UserPageResponse response = new UserPageResponse();
        response.setTotal(userPage.getTotalElements());
        response.setCurrentPage(currentPage);
        response.setPageSize(pageSize);

        List<User> users = userPage.getContent();
        users.forEach(user -> user.setPassword(null));
        response.setList(users);

        return ApiResponse.success(response);
    }

    public User register(User user) {
        // 检查邮箱是否已存在
        if (userRepository.findByEmail(user.getEmail()).isPresent()) {
            throw new RuntimeException("邮箱已存在");
        }

        // 检查用户名是否已存在
        if (userRepository.findByUsername(user.getUsername()).isPresent()) {
            throw new RuntimeException("用户名已存在");
        }

        // 设置默认值
        user.setDepartmentId(0L);
        user.setRoles(0);
        user.setJobs(0);
        user.setCreatorId(0L);

        // 加密密码
        user.setPassword(passwordEncoder.encode(user.getPassword()));
        user.setStatus(1);
        user.setCreatedAt(System.currentTimeMillis() / 1000);
        user.setUpdatedAt(System.currentTimeMillis() / 1000);

        // 保存用户
        User savedUser = userRepository.save(user);

        // 清除密码后返回
        savedUser.setPassword(null);
        return savedUser;
    }

    public List<User> findByDepartmentId(Long departmentId) {
        logger.info("查询部门用户列表，部门ID: {}", departmentId);
        return userRepository.findByDepartmentIdAndNormalStatus(departmentId);
    }

    public ApiResponse<User> findTeacherByEmail(String email) {
        if (email == null || email.trim().isEmpty()) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "邮箱不能为空", null);
        }

        User user = userRepository.findByEmail(email).orElse(null);
        if (user == null) {
            return new ApiResponse<>(ErrorCode.USER_NOT_FOUND, "用户不存在", null);
        }

        if (user.getStatus() != 1) {
            return new ApiResponse<>(ErrorCode.INVALID_PARAM, "用户状态异常", null);
        }

        return ApiResponse.success(user);
    }

    public ApiResponse<String> batchImportUsers(List<UserImportDTO> userList) {
        if (userList == null || userList.isEmpty()) {
            return new ApiResponse<>(ErrorCode.PARAM_ERROR, "导入数据不能为空", null);
        }

        StringBuilder errorMsg = new StringBuilder();
        int successCount = 0;

        for (int i = 0; i < userList.size(); i++) {
            UserImportDTO dto = userList.get(i);
            try {
                // 基本数据验证
                if (StringUtils.isBlank(dto.getUsername()) ||
                        StringUtils.isBlank(dto.getEmail()) ||
                        StringUtils.isBlank(dto.getPassword()) ||
                        StringUtils.isBlank(dto.getDepartmentName()) ||
                        StringUtils.isBlank(dto.getRoleAndJob())) {
                    errorMsg.append(String.format("第%d行数据不完整;", i + 2));
                    continue;
                }

                // 检查邮箱是否已存在
                if (userRepository.findByEmail(dto.getEmail()).isPresent()) {
                    errorMsg.append(String.format("第%d行邮箱已存在;", i + 2));
                    continue;
                }

                // 查找部门ID
                Department department = departmentRepository.findByName(dto.getDepartmentName());
                logger.info("查找部门：{}，结果：{}", dto.getDepartmentName(), department);
                if (department == null) {
                    errorMsg.append(String.format("第%d行部门不存在;", i + 2));
                    continue;
                }

                // 创建新用户
                User user = new User();
                user.setUsername(dto.getUsername());
                user.setEmail(dto.getEmail());
                user.setPassword(passwordEncoder.encode(dto.getPassword()));
                user.setDepartmentId(department.getId());
                user.setRoles(0); // 默认角色
                user.setJobs(0); // 默认工作
                user.setStatus(1); // 设置状态为正常
                user.setCreatedAt(System.currentTimeMillis() / 1000);
                user.setUpdatedAt(System.currentTimeMillis() / 1000);

                try {
                    userRepository.save(user);
                    successCount++;
                } catch (Exception e) {
                    logger.error("保存用户失败", e);
                    errorMsg.append(String.format("第%d行保存失败: %s;", i + 2, e.getMessage()));
                }
            } catch (Exception e) {
                logger.error("导入第{}行数据失败", i + 2, e);
                errorMsg.append(String.format("第%d行导入失败;", i + 2));
            }
        }

        String resultMsg = String.format("成功导入%d条数据", successCount);
        if (errorMsg.length() > 0) {
            resultMsg += "。错误信息：" + errorMsg;
        }

        // 如果没有成功导入任何数据，返回错误状态
        if (successCount == 0) {
            return new ApiResponse<>(ErrorCode.PARAM_ERROR, resultMsg, null);
        }

        return ApiResponse.success(resultMsg);
    }
}
